//
//     Copyright (C) 2003 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//     File: ImageWindow.cpp
//

#include <SophiaFramework.hpp>
#include "ImageWindow.hpp"
#include "Cartoon.hpp"

//
// Constructor definition.
//
ImageWindow::ImageWindow(SFRApplicationPtr papp, SFXRectangleConstRef rect, SFBBitmapSmpConstRef bmp)
    : SFRPlainWindow(papp, rect)
{
    // Various handlers are registered here
    RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnDraw));
    RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey));

    // Initialization of displayed bitmap
    m_bitmap = bmp;

    // Determining scale ratio of the bitmap
    Float64 bmp_aspect_ratio;
    Float64 dev_aspect_ratio;
    AEEDeviceInfo devinfo;
    AEEBitmapInfo bmpinfo;
    SFBShellSmp shell = SFBShell::GetInstance();

    m_bitmap->GetInfo(&bmpinfo);
    m_bitmap_size.SetWidth(static_cast<SInt16>(bmpinfo.cx));
    m_bitmap_size.SetHeight(static_cast<SInt16>(bmpinfo.cy));

    shell->GetDeviceInfo(&devinfo);
    m_devsize.SetWidth(devinfo.cxScreen);
    m_devsize.SetHeight(devinfo.cyScreen);

    bmp_aspect_ratio = static_cast<Float64>(m_bitmap_size.GetWidth()) / static_cast<Float64>(m_bitmap_size.GetHeight());
    dev_aspect_ratio = static_cast<Float64>(m_devsize.GetWidth()) / static_cast<Float64>(m_devsize.GetHeight());

    Bool isheight_fit;
    if (dev_aspect_ratio >= bmp_aspect_ratio) {
        m_scale_ratio = static_cast<Float64>(m_devsize.GetHeight() / m_bitmap_size.GetHeight());
        isheight_fit = true;
    }
    else {
        m_scale_ratio = static_cast<Float64>(m_devsize.GetWidth()) / static_cast<Float64>(m_bitmap_size.GetWidth());
        isheight_fit = false;
    }

    //  Coordinates at the forwarding destination are specified
    m_transfered_point.SetX(static_cast<SInt16>(m_bitmap_size.GetWidth() * (m_scale_ratio - 1.0) / 2.0));
    m_transfered_point.SetY(static_cast<SInt16>(m_bitmap_size.GetHeight() * (m_scale_ratio - 1.0) / 2.0));
    if (isheight_fit) {
        m_transfered_point.AddX(static_cast<SInt16>((m_devsize.GetWidth() - m_bitmap_size.GetWidth() * m_scale_ratio) / 2.0));
    }
    else {
        m_transfered_point.AddY(static_cast<SInt16>((m_devsize.GetHeight() - m_bitmap_size.GetHeight() * m_scale_ratio) / 2.0));
    }
}

//
// Destructor definition.
// Nothing special is done here.
//
ImageWindow::~ImageWindow()
{
    return;
}

//
//  Drawing routine.
//
HANDLER_IMPLEMENT_VOIDRENDER(ImageWindow, OnDraw, graphic)
{
    SFBShellSmp        shell = SFBShell::GetInstance();
    SFBBitmapSmp       pSourceBitmap;
    SFBTransformSmp    pTransform;
    AEETransformMatrix           matrix;

    //  The window is painted with a white background color
    graphic->FillRectangle(GetContentWorld(), SFXRGBColor(255, 255, 255, 0));

    //  Matrix for modification
    matrix.A = static_cast<SIntN>(m_scale_ratio * 256.0);
    matrix.B = 0;
    matrix.C = 0;
    matrix.D = static_cast<SIntN>(m_scale_ratio * 256.0);

    //  The bitmap is enlarged and displayed
    pSourceBitmap = graphic->CreateBitmap(m_bitmap_size);
    pSourceBitmap->BltIn(SFXRectangle(0, 0, m_bitmap_size), m_bitmap, SFXGrid(0, 0));

    pTransform = SFBTransform::NewInstance(graphic->GetDestination());
    pTransform->TransformBltComplex(
        SFXGrid(m_transfered_point.GetX(), m_transfered_point.GetY()),
        pSourceBitmap, SFXRectangle(0, 0, m_bitmap_size), matrix);
}

//
//      Key handler.
//
HANDLER_IMPLEMENT_BOOLEVENT(ImageWindow, OnKey, event)
{
    UInt16 key = event.GetP16();
    switch (key) {
    case AVK_SELECT:
    case AVK_CLR:
        // Terminate window
        Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
        return true;
    }
    return true;
}
